CPU 120
Option explicit
Option default none

Const setup_button=1
Const time_button=2
Const pos_button=3
Const trig_button=4
Const clear_button=5

Const usec_spinbox=6
Const ch1_pos_spinbox=7
Const ch2_pos_spinbox=8
Const ch1_trig_spinbox=9
Const ch2_trig_spinbox=10

Dim setup_active As float ' flags for setup state
Dim screen_state As float ' current screen
screen_state=1 'default screen - Time
Dim setup_time As integer ' , copy of timer value when entering setup - used for timeout

Dim integer i,j=0,toggle,channel1buff(MM.HRes/2),channel2buff(MM.HRes/2),microsecondsperdivision=5000
Dim integer y1pos=330, y2pos=150, c1mvperdivision=1000, c2mvperdivision=1000, c1trig=0, c2trig=0

GUI interrupt pen_down,pen_up ' touch INT handlers
GUI default hidden

Font 4
GUI button time_button,"Time",670,2,MM.FontWidth*7,MM.FontHeight*2,RGB(yellow),RGB(gray)
GUI spinbox usec_spinbox,400,5,MM.FontWidth*14,MM.FontHeight*2,RGB(yellow),RGB(black),10,100,100000
CtrlVal(usec_spinbox)=microsecondsperdivision ' startup val

GUI button pos_button,"Pos",670,2,MM.FontWidth*7,MM.FontHeight*2,RGB(yellow),RGB(gray)
GUI spinbox ch1_pos_spinbox,160,5,MM.FontWidth*14,MM.FontHeight*2,RGB(yellow),RGB(black),1,1,450
CtrlVal(ch1_pos_spinbox)=y1pos
GUI spinbox ch2_pos_spinbox,400,5,MM.FontWidth*14,MM.FontHeight*2,RGB(yellow),RGB(black),1,1,450
CtrlVal(ch2_pos_spinbox)=y2pos


'End


If npins()=100 Then
  SetPin 25,ain ' chan 1
  SetPin 21,ain ' chan 2
EndIf
If npins()=64 Then
  SetPin 16,ain ' chan 1
  SetPin 12,ain ' chan 2
EndIf
If npins()=44 Then
  SetPin 19,ain ' chan 1
  SetPin 10,ain ' chan 2
EndIf
If npins()=28 Then
  SetPin 2,ain ' chan 1
  SetPin 23,ain ' chan 2
EndIf

'SetPin 16,ain 'Channel 1
'SetPin 12,ain 'Channel 2

Colour RGB(white),RGB(blue)
CLS
Font 2
'
'  Starting code for a Micromite based oscilloscope - needs a GUI (volunteers?)
'  Runs on MM+ with a 800x480 SSD1963 display
'  Uses pin 16 for channel 0 and pin 12 for channel 1
'  Maximum conversion rate is approximately 1.6 Megasamples / second on each channel at 10-bits resolution
'  The CFunction will automatically compensate for CPU speed but obviously the higher the speed the better
'
'
'  Parameters for call to CFunction "scope"
'
'      Returns the scaling factor used for the display or a large number if triggering is enabled and has timed out
'      A scale factor of 1 means every x-coordinate has a unique value
'
'      microsecondsperdivision: specifies the number of microseconds to scale the display for each graticle line
'         note: durations less than 100uSec will use scaling. i.e.every other x-coordinate will be missed if scale = 2
'      toggle: initialised to 0, used by the CFunction to control its double buffering
'      channel1buff: buffer used for channel 1, used as 2 buffers of 800 x 16-bit words
'      channel2buff: buffer used for channel 2, used as 2 buffers of 800 x 16-bit words
'      y-position for channel 1: specifies the screen position of the zero volts on channel 1 in rows from the bottom of the display
'           use any negative number to turn off the channel
'      y-position for channel 2: specifies the screen position of the zero volts on channel 2 in rows from the bottom of the display
'           use any negative number to turn off the channel
'      trigger for channel 1
'           specifies the ADC reading and direction to trigger the trace for channel 1 (-1024 to +1023)
'           a zero value means triggering is disabled for this channel
'           a positive value triggers when the ADC reading passes the trigger value in a positive going sense
'           a negative value triggers when the ADC reading passes the absolute trigger value in a negative going sense
'      trigger for channel 2
'           specifies the ADC reading and direction to trigger the trace for channel 2 (-1024 to +1023)
'           a zero value means triggering is disabled for this channel
'           a positive value triggers when the ADC reading passes the trigger value in a positive going sense
'           a negative value triggers when the ADC reading passes the absolute trigger value in a negative going sense
'      NB: if both trigger levels are set to zero the scope will free run
'      millivolts per division for channel 1 based on VDD=3.3V
'      millivolts per division for channel 2 based on VDD=3.3V
'

Do
  i= scope(microsecondsperdivision,toggle,channel1buff(),channel2buff(),y1pos, y2pos, c1trig, c2trig ,c1mvperdivision,c2mvperdivision)
  If i>1 And i<1024 And j=0 Then
      Print "Warning reduced resolution x"+Str$(i)
      j=1
  EndIf
  If i>1024 Then
      Print "No trigger"
  EndIf
'  If Not setup_active Then
    Text 2,2,"T:"+Str$(microsecondsperdivision)+"us/div"
    If y1pos>=0 Then Text 2,24,"C1:"+Str$(c1mvperdivision)+"mv/div"
    If y2pos>=0 Then Text 2,46,"C2:"+Str$(c2mvperdivision)+"mv/div"
'  EndIf
  If setup_active And setup_time+3000<Timer Then
  setup_active=0
  GUI hide all
  EndIf

Loop

Sub pen_down
If setup_active Then
    i= scope(-microsecondsperdivision,toggle,channel1buff(),channel2buff(),y1pos, y2pos, c1trig, c2trig ,c1mvperdivision,c2mvperdivision)
    setup_time=Timer
    If screen_state=1 Then
      microsecondsperdivision=CtrlVal(usec_spinbox) ' update setting value
    EndIf
    If screen_state=2 Then
      y1pos=CtrlVal(ch1_pos_spinbox)
      y2pos=CtrlVal(ch2_pos_spinbox)
    EndIf
EndIf
'Text 100,100,"Pen Dn "+Str$(Touch(ref))
End Sub

Sub pen_up
    setup_time=Timer
    If Not setup_active Then
      setup_active=1
      show_setup_screen
    EndIf
    Select Case Touch(lastref)
      Case time_button
        GUI hide all
        screen_state=2
        show_setup_screen
      Case pos_button
        GUI hide all
        screen_state=1
        show_setup_screen
      End Select
'Text 100,100,"Pen Up "+Str$(Touch(lastref))
End Sub

Sub show_setup_screen
    If screen_state=1 Then show_screen_1
    If screen_state=2 Then show_screen_2
    If screen_state=3 Then show_screen_3
End Sub

Sub show_screen_1
    GUI show time_button
    GUI show usec_spinbox
End Sub

Sub show_screen_2
    GUI show ch1_pos_spinbox
    GUI show ch2_pos_spinbox
    GUI show pos_button
End Sub

Sub show_screen_3
End Sub

End
'
CFunction npins 'returns the number of pins on the chip in use.
    00000000
    3C03BF81 8C65F220 3C020661 7CA5D800 3444A053 50A40009 2402002C 8C64F220
    2443A053 7C84D800 00832026 2402001C 2403002C 0064100A 3C03BF81 8C66F220
    8C64F220 3C030580 3463A053 7CC6D800 00C33026 24050040 7C84D800 24631000
    10830006 00A6100A 00022FC3 00402021 00801021 03E00008 00A01821 24040064
    00002821 00801021 03E00008 00A01821
End CFunction

CFunction scope
    00000000
    3C029D00 8C480094 27BDFF70 8C430000 AFB40078 8D140000 8C480098 8C6A0000
    AFB1006C 8D110000 3C081062 25084DD3 01480019 001458C0 00141140 01621021
    00004010 00026080 3C03BF81 004C1021 00086182 8FA800B0 8C69F220 AFB20070
    3C0A0661 8D120000 8FA800B4 7D29D800 354BA053 AFB30074 AFBF008C AFBE0088
    AFB70084 AFB60080 AFB5007C AFB00068 8FAD00AC 2631FFFF 112B02BA 8D130000
    8C63F220 254AA053 7C63D800 006A5026 2409001C 2403002C 006A480A 3C03BF81
    8C6BF220 3C0A0580 8C68F220 354AA053 7D6BD800 016A5826 24030040 7D08D800
    254A1000 110A02A0 006B480A 29280033 24090028 0068480A 8C880004 0500029E
    8C830000 24040001 AFA40020 8CAA0000 8CA40004 01444025 11000205 0014F040
    00144040 00E85821 AFAB0018 00C84021 AFA6002C AFA70030 000A5027 00042027
    ACAA0000 ACA40004 706C1802 10000002 24100001 00808021 0050001B 020001F4
    00002812 0065001B 00A001F4 0000B012 02C9282B 14A0FFF7 26040001 8FA200A8
    8C460000 14C000AA 241EFFFF 8DA70000 0007F00B 00063023 7C063620 18E000AB
    AFA00024 8FA40020 2403FFFF 7C073E20 108000AB AFA30028 3C021000 3C19BF88
    AF221064 3C05BF88 34028000 3C0BBF81 ACA01068 3C03BF81 AD629004 8C65F220
    3C020661 7CA5D800 3444A053 50A40008 2402002C 8C64F220 2443A053 7C84D800
    00832026 2402001C 2403002C 0064100A 3C03BF81 8C69F220 8C64F220 3C030580
    3463A053 7D29D800 01234826 24050040 7C84D800 24631000 10830254 00A9100A
    28420033 10400251 3C030C00 3C02BF81 AC439040 3C02BF88 AC406118 3C02BF88
    AC406108 3C03FFFF 3C02BF81 AC439050 24030200 3C02BF81 AC439020 24030007
    3C02BF81 AC439010 340380E4 3C02BF81 AC439000 3C02BF81 34038000 AC439008
    3C0DBF88 3C020008 AF221064 3C15BF81 ADA01068 8EA29010 7C4201C0 1040FFFD
    3C0ABF81 3C09BF81 8D429070 8D229080 8EA29010 30420080 1440FFFD 3C0DBF81
    0290001B 020001F4 3C0CBF81 8DA490F0 8D849100 8D459070 8D249080 24A50001
    A5050000 8FA50018 24840001 A4A40000 00001812 0000A012 40824800 1280022B
    00002821 0000B821 00007021 3C0B9D00 8EA29010 7C4201C0 1040FFFD 00000000
    40024800 0056001B 02C001F4 00007812 02EF102B 1040000B 00000000 8D449070
    8D229080 13C00179 8FBF0018 00177840 010FC021 03EF7821 A7040000 26F70001
    A5E20000 8EA29010 30420080 1440FFFD 00000000 40024800 0056001B 02C001F4
    0000C012 02F8102B 1040000C 02E3102B 8DA490F0 8D829100 13C0017F 8FBF0018
    00177840 010FC021 03EF7821 A7040000 26F70001 A5E20000 02E3102B 1440FFD4
    00000000 3C03004C 24634B40 00A3282B 3C031000 AF231064 3C020008 3C1FBF88
    34038000 3C04BF81 AFE01068 AC839004 AF221064 AFE21068 14A00012 3C029D00
    3C12004C 36524B40 100000E8 00009821 18C001F3 2402FFFF 8DA70000 7C063620
    0000F021 1CE0FF57 AFA20024 8FA40020 00073823 7C073E20 1480FF57 AFA00028
    3C029D00 8C420098 24150080 8C430000 3C028888 34428889 00620019 00001010
    000211C2 02A2001B 004001F4 8FA200A0 8C4B0000 0000A812 05600065 0015B043
    2E820002 14400062 24020001 8FBE002C AFB00028 AFB3002C 0000B821 3C0A9D00
    24033057 AFB40024 00409821 10000006 01008021 8FA20024 12620052 26100002
    8FA400A0 8C8B0000 87C70002 87C50000 70E33802 70A32802 00F2001B 024001F4
    24020001 AFA20010 8FA40028 240200FF AFA20014 8D420050 02E0A021 02E4B821
    AFA30060 AFAA005C 02802021 02E03021 26730001 27DE0002 00003812 00B2001B
    024001F4 00F63821 00002812 00B62821 00B5001B 02A001F4 00002812 00F5001B
    02A001F4 02252823 00AB2823 00003812 02273823 0040F809 00EB3823 8FA20020
    8FA30060 1040FFD3 8FAA005C 86050002 86080000 70A32802 71034002 00B2001B
    024001F4 8FA200A0 3C0B00FF 24040001 356BFFFF 8C470000 AFA40010 AFAB0014
    8D420050 02802021 02E03021 26100002 00002812 0112001B 024001F4 00B62821
    00004012 01164021 0115001B 02A001F4 00004012 00B5001B 02A001F4 02282823
    00A72823 00004812 02294823 0040F809 01273823 8FA20024 8FAA005C 1662FFB0
    8FA30060 8FB40024 8FB3002C 8FB00028 8FA500A4 8CAB0000 05600061 2E820002
    1440005F 8FA30018 AFB4001C 8FB70030 00009021 241E0001 3C099D00 24023057
    0060A021 10000006 AFB00018 8FA3001C 13C30052 26940002 8FA400A4 8C8B0000
    86E70002 86E50000 70E23802 70A22802 00F3001B 026001F4 24030001 AFA30010
    8FA40018 240300FF AFA30014 8D230050 02408021 02449021 AFA20034 AFA90058
    02002021 02403021 27DE0001 26F70002 00003812 00B3001B 026001F4 00F63821
    00002812 00B62821 00B5001B 02A001F4 00002812 00F5001B 02A001F4 02252823
    00AB2823 00003812 02273823 0060F809 00EB3823 8FA30020 8FA20034 1060FFD3
    8FA90058 86850002 86880000 70A22802 71024002 00B3001B 026001F4 8FA300A4
    3C0B00FF 24040001 356BFFFF 8C670000 AFA40010 AFAB0014 8D230050 02002021
    02403021 26940002 00002812 0113001B 026001F4 00B62821 00004012 01164021
    0115001B 02A001F4 00005012 00B5001B 02A001F4 022A2823 00A72823 00004012
    02284023 0060F809 01073823 8FA3001C 8FA90058 17C3FFB0 8FA20034 8FB00018
    02009021 00009821 3C029D00 8C420098 8C420000 10400016 3C150080 00008021
    3C149D00 24160001 36B58080 8E820094 02002821 8C460000 AFB60010 AFB50014
    8E820050 02003821 00002021 0040F809 24C6FFFF 8E820098 8C420000 000218C2
    02038021 0202102B 5440FFF1 8E820094 3C029D00 8C420094 8C460000 10C00018
    3C160080 3C15CCCC 00008021 3C149D00 24170001 36D68080 36B5CCCD AFB70010
    AFB60014 8E820050 02003021 02002021 00002821 0040F809 02203821 8E820094
    8C460000 00D50019 00001010 000210C2 02028021 0206102B 5440FFF1 AFB70010
    3C140080 36948080 24150001 3C109D00 AFB50010 AFB40014 8E020050 02202821
    02203821 00002021 0040F809 24C6FFFF 8E020094 02203821 8C440000 AFB50010
    AFB40014 8E020050 2484FFFF 00002821 0040F809 00803021 8FBF008C 02401021
    02601821 8FBE0088 8FB70084 8FB60080 8FB5007C 8FB40078 8FB30074 8FB20070
    8FB1006C 8FB00068 03E00008 27BD0090 00FE7821 00DEF021 AFAF0030 AFBE002C
    00C04021 1000FDFC AFA70018 10C00006 8FAF0024 11E0006E 850F0000 01E6782A
    55E00096 0086F02B 10E0000D 8FAF0018 8FAF0028 11E00058 8FBF0018 8FB80018
    870F0000 01E7782A 11E00005 8FAF0018 0047C02B 001E7827 01F8F00A 8FAF0018
    A5040000 01C02021 A5E20000 40844800 1000FE74 24A50001 10C00006 8FAF0024
    11E0005A 850F0000 01E6782A 55E00077 0086F02B 10E0000A 8FAF0018 8FAF0028
    11E00059 8FBF0018 8FB80018 870F0000 01E7782A 15E0006A 0047C02B 8FAF0018
    A5040000 A5E20000 01C02021 40844800 8D620090 3C18004C 24A50001 80440017
    37184B40 00B8102B 1080FE6B 0062B80A 8D620020 AFA30060 AFA50054 AFA60048
    AFA7004C AFA80050 AFA90058 AFAA005C AFAB0044 AFAC0038 AFAD0034 AFAE0040
    0040F809 AFB9003C 8FA30060 8FA50054 8FA60048 8FA7004C 8FA80050 8FA90058
    8FAA005C 8FAB0044 8FAC0038 8FAD0034 8FAE0040 1440FE50 8FB9003C 3C031000
    AF231064 3C020008 3C08BF88 3C12004C 34038000 3C0BBF81 AD001068 36524B40
    AD639004 00009821 AF221064 AD021068 1000FF3E 3C029D00 87EF0000 00EF782A
    11E0FFAF 8FAF0018 00E2C02B 001E7827 01F8F00A 8FAF0018 A5040000 01C02021
    A5E20000 40844800 1000FE1E 24A50001 00CF782A 11E0FF94 241FFFFF 00C4F02B
    001EF80B 1000FF90 03E0F021 00CF782A 11E0FFA8 241FFFFF 00C4F02B 001EF80B
    1000FFA4 03E0F021 87EF0000 00EF782A 11E0FFAB 8FAF0018 00E2C02B 001E7827
    1000FFA6 01F8F00A 8C880004 24090040 0501FD64 8C830000 00031823 1000FD63
    AFA00020 1000FD4D 2409002C 3C030400 3C02BF81 AC439040 1000FDAF 3C02BF88
    001E7827 1000FF95 01F8F00A 2418FFFF 001EC00B 1000FF87 0300F021 2418FFFF
    001EC00B 1000FF68 0300F021 3C031000 AF231064 3C03BF88 AC601068 3C020008
    34038000 3C04BF81 3C05BF88 AC839004 AF221064 ACA21068 1000FE1A 3C029D00
    8DA70000 1000FD63 0000F021
End CFunction                              